from .utils import wait_for_selector

def interrupt_from_menu(notebook):
    # Click interrupt button in kernel menu
    notebook.browser.find_element_by_id('kernellink').click()
    wait_for_selector(notebook.browser, '#int_kernel', single=True).click()

def interrupt_from_keyboard(notebook):
    notebook.body.send_keys("ii")


def test_interrupt(notebook):
    """ Test the interrupt function using both the button in the Kernel menu and the keyboard shortcut "ii"

        Having trouble accessing the Interrupt message when execution is halted. I am assuming that the
        message does not lie in the "outputs" field of the cell's JSON object. Using a timeout work-around for
        test with an infinite loop. We know the interrupt function is working if this test passes.
        Hope this is a good start.
    """

    text = ('import time\n'
            'for x in range(3):\n'
            '   time.sleep(1)')

    notebook.edit_cell(index=0, content=text)

    for interrupt_method in (interrupt_from_menu, interrupt_from_keyboard):
        notebook.clear_cell_output(0)
        notebook.to_command_mode()
        notebook.execute_cell(0)

        interrupt_method(notebook)

        # Wait for an output to appear
        output = wait_for_selector(notebook.browser, '.output_subarea', single=True)
        assert 'KeyboardInterrupt' in output.text



from tornado import web
from ..base.handlers import IPythonHandler
from ..services.kernelspecs.handlers import kernel_name_regex

class KernelSpecResourceHandler(web.StaticFileHandler, IPythonHandler):
    SUPPORTED_METHODS = ('GET', 'HEAD')

    def initialize(self):
        web.StaticFileHandler.initialize(self, path='')

    @web.authenticated
    def get(self, kernel_name, path, include_body=True):
        ksm = self.kernel_spec_manager
        try:
            self.root = ksm.get_kernel_spec(kernel_name).resource_dir
        except KeyError as e:
            raise web.HTTPError(404,
                                u'Kernel spec %s not found' % kernel_name) from e
        self.log.debug("Serving kernel resource from: %s", self.root)
        return web.StaticFileHandler.get(self, path, include_body=include_body)

    @web.authenticated
    def head(self, kernel_name, path):
        return self.get(kernel_name, path, include_body=False)

default_handlers = [
    (r"/kernelspecs/%s/(?P<path>.*)" % kernel_name_regex, KernelSpecResourceHandler),
]


"""Test HTML utils"""

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import ctypes
import os
import sys

import pytest

from traitlets.tests.utils import check_help_all_output
from notebook.utils import url_escape, url_unescape, is_hidden, is_file_hidden
from ipython_genutils.py3compat import cast_unicode
from ipython_genutils.tempdir import TemporaryDirectory
from ipython_genutils.testing.decorators import skip_if_not_win32


def test_help_output():
    """jupyter notebook --help-all works"""
    # FIXME: will be notebook
    check_help_all_output('notebook')


def test_url_escape():

    # changes path or notebook name with special characters to url encoding
    # these tests specifically encode paths with spaces
    path = url_escape('/this is a test/for spaces/')
    assert path == '/this%20is%20a%20test/for%20spaces/'

    path = url_escape('notebook with space.ipynb')
    assert path == 'notebook%20with%20space.ipynb'

    path = url_escape('/path with a/notebook and space.ipynb')
    assert path == '/path%20with%20a/notebook%20and%20space.ipynb'
    
    path = url_escape('/ !@$#%^&* / test %^ notebook @#$ name.ipynb')
    assert path == '/%20%21%40%24%23%25%5E%26%2A%20/%20test%20%25%5E%20notebook%20%40%23%24%20name.ipynb'

def test_url_unescape():

    # decodes a url string to a plain string
    # these tests decode paths with spaces
    path = url_unescape('/this%20is%20a%20test/for%20spaces/')
    assert path == '/this is a test/for spaces/'

    path = url_unescape('notebook%20with%20space.ipynb')
    assert path == 'notebook with space.ipynb'

    path = url_unescape('/path%20with%20a/notebook%20and%20space.ipynb')
    assert path == '/path with a/notebook and space.ipynb'

    path = url_unescape(
        '/%20%21%40%24%23%25%5E%26%2A%20/%20test%20%25%5E%20notebook%20%40%23%24%20name.ipynb')
    assert path == '/ !@$#%^&* / test %^ notebook @#$ name.ipynb'

def test_is_hidden():
    with TemporaryDirectory() as root:
        subdir1 = os.path.join(root, 'subdir')
        os.makedirs(subdir1)
        assert is_hidden(subdir1, root) == False
        assert is_file_hidden(subdir1) == False

        subdir2 = os.path.join(root, '.subdir2')
        os.makedirs(subdir2)
        assert is_hidden(subdir2, root) == True
        assert is_file_hidden(subdir2) == True
        # root dir is always visible
        assert is_hidden(subdir2, subdir2) == False

        subdir34 = os.path.join(root, 'subdir3', '.subdir4')
        os.makedirs(subdir34)
        assert is_hidden(subdir34, root) == True
        assert is_hidden(subdir34) == True

        subdir56 = os.path.join(root, '.subdir5', 'subdir6')
        os.makedirs(subdir56)
        assert is_hidden(subdir56, root) == True
        assert is_hidden(subdir56) == True
        assert is_file_hidden(subdir56) == False
        assert is_file_hidden(subdir56, os.stat(subdir56)) == False

@pytest.mark.skipif(sys.platform != "win32", reason="run on windows only")
def test_is_hidden_win32():
    with TemporaryDirectory() as root:
        root = cast_unicode(root)
        subdir1 = os.path.join(root, u'subdir')
        os.makedirs(subdir1)
        assert not is_hidden(subdir1, root)
        r = ctypes.windll.kernel32.SetFileAttributesW(subdir1, 0x02)
        print(r)
        assert is_hidden(subdir1, root)
        assert is_file_hidden(subdir1)

